#!/usr/bin/env python3

# Copyright © 2018 Broadcom. All rights reserved. The term "Broadcom"
# refers to Broadcom Inc. and/or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may also obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""

:mod:`port_fault_delay_state_set` - PyFOS util for specific port op use case.
***********************************************************************************
The :mod:`port_fault_delay_state_set` provides fora  specific port op use case.

This module is a stand-alone script that can be used to set/reset  \
fault-delay config for a port.

* Infrastructure options:
    * -L=<login>: Login ID. If not provided, an interactive
        prompt will request one.
    * -P=<password>: Password. If not provided, an interactive
        prompt will request one.
    * -i=<IP address>: IP address.
    * -n=<port name>: <slot>/<port> name of the port.
    * -u=<user name>: string name to be assigned to switch.
    * -f=<VFID>: VFID or -1 if VF is disabled. If unspecified,
        VFID of 128 is assumed.

* Util script options:
    -n,--name=NAME                                   Port in slot/port.
    --fault_delay_enabled=FAULT_DELAY_ENABLED \
                                   Set "falut_delay-port-enabled" <0|1>


* Outputs:
    * Python dictionary content with RESTCONF response data.

"""

import pyfos.pyfos_auth as pyfos_auth
from pyfos.pyfos_brocade_fibrechannel import fibrechannel
import pyfos.pyfos_util as pyfos_util
import sys
import pyfos.utils.brcd_util as brcd_util


def validate(fcObject):
    if fcObject.peek_name() is None or \
       fcObject.peek_fault_delay_enabled() is None:
        return 1
    return 0


def main(argv):
    filters = ["name", "fault_delay_enabled"]
    inputs = brcd_util.parse(argv, fibrechannel, filters, validate)

    fcObject = inputs['utilobject']

    session = brcd_util.getsession(inputs)

    result = fcObject.patch(session)
    pyfos_util.response_print(result)

    pyfos_auth.logout(session)


if __name__ == "__main__":
    main(sys.argv[1:])
